package decatur.de.rigorous.plot;

import java.awt.*;
import java.applet.Applet;

import decatur.de.rigorous.numutil.*;
import decatur.de.rigorous.parse.*;

class MyPanel extends Panel {
	private GridBagConstraints gbc;
	private GridBagLayout layout;

	public TextField xRangeField, yRangeField, nodesField;
	public TextArea funcArea, messageArea;
	Button eval, print;
	Choice choice;

	public Component add(Component c) {
		layout.setConstraints(c, gbc);
		super.add(c);
		return c;
	}

	public MyPanel() {
		super();

		gbc = new GridBagConstraints();
		gbc.insets = new Insets(3, 3, 3, 3);
		gbc.ipadx = gbc.ipady = 5;
		layout = new GridBagLayout();
		setLayout(layout);

		choice = new Choice();
		add(choice);
		
		eval = new Button("Evaluate");
		add(eval);
		
		print = new Button("Print");
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		add(print);

		funcArea = new TextArea("[0.5,0.55]*x^3-x^2", 4, 25);
		GuiLabelComponent func = new GuiLabelComponent("Function f(x)",
				funcArea);
		gbc.gridwidth = 3;
		gbc.gridheight = 2;
		add(func);

		xRangeField = new TextField("", 15);
		GuiLabelComponent xRange = new GuiLabelComponent("x-Range", xRangeField);
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.gridheight = 1;
		add(xRange);

		yRangeField = new TextField("", 15);
		GuiLabelComponent yRange = new GuiLabelComponent("y-Range", yRangeField);
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		add(yRange);

		messageArea = new TextArea("Message area", 4, 25);
		GuiLabelComponent message = new GuiLabelComponent("Messages",
				messageArea);
		gbc.gridwidth = 3;
		add(message);

		nodesField = new TextField("", 15);
		GuiLabelComponent nodes = new GuiLabelComponent("Nodes", nodesField);
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		add(nodes);
		// validate ();
	}

	public void Bounds(String infx, String supx, String infy, String supy) {
		xRangeField.setText("[" + infx + ", " + supx + "]");
		yRangeField.setText("[" + infy + ", " + supy + "]");
	}

	public Insets insets() {
		return new Insets(4, 4, 5, 5);
	}

	public void paint(Graphics g) {
		Dimension d = getSize();
		Color bg = getBackground();
		g.setColor(bg);
		g.draw3DRect(0, 0, d.width - 1, d.height - 1, true);
		g.draw3DRect(3, 3, d.width - 7, d.height - 7, false);
	}
}

public class RigorousGraphing extends Applet {
	MyPanel cp;
	Expression expression = null;
	String funcdef = "";
	String[][] models = {
			{ "cubic", "[0.5,0.55]*x^3-2*x", "-3", "3", "-10", "10", "40" },
			{ "trig1", "sin(x*[1,1.03])", "0", "40", "-2", "2", "150" },
			{ "singular", "1/(x^2-0.2)", "-2", "2", "-20", "20", "100" },
			{ "trig2", "x*sin(1/x)", "0", "0.1", "-3", "3", "80" },
			{ "identity", "sin(x)^2+cos(x)^2", "0", "20", "0", "2", "80" } };

	RootFinder rootfinder;
	Interval xrange, yrange;
	SoAxisCanvas canvas;
	FlowLayout fl;

	public void init() {
		fl = new FlowLayout();
		setLayout(fl);

		xrange = new Interval();
		yrange = new Interval();
		rootfinder = new RootFinder();
		canvas = new SoAxisCanvas(400, 400);
		cp = new MyPanel();
		for (int i = 0; i < models.length; i++)
			cp.choice.addItem(models[i][0]);
		add(canvas);
		add(cp);
		selectModel(0);
		validate();
	}

	public boolean action(Event evt, Object what) {
		if (evt.target == cp.eval) {
			reset();
			return true;
		} else if (evt.target == cp.choice) {
			selectModel(cp.choice.getSelectedIndex());
			return true;
		} else if (evt.target == cp.print) {
			canvas.print((java.io.PrintStream) null);
		}

		return false;
	}

	public boolean mouseUp(Event evt, int xdummy, int ydummy) {
		if (evt.arg instanceof SbVec2f) {
			SbVec2f p = (SbVec2f) evt.arg;
			try {
				Interval x = rootfinder.find(p.x, expression);
				cp.messageArea.setText("Enclosure of root is\n");
				cp.messageArea.append("[" + x.inf + ",\n " + x.sup + "]");

				SbVec2f res = canvas.getResolution();
				x.inflate(res.x * 2.0);
				SoRectangle r = new SoRectangle();
				r.x = (float) x.inf;
				r.width = (float) x.width();
				r.y = -5.0f * res.y;
				r.height = 10.0f * res.y;
				canvas.addNode(r);
				canvas.touch();
			} catch (ArithmeticException e) {
				cp.messageArea.setText("" + e);
			}
			return true;
		} else if (evt.arg instanceof SbBox2f) {
			SbBox2f box = (SbBox2f) evt.arg;
			box.normalize();

			double dx = box.width * 0.01;
			double dy = box.height * 0.01;

			cp.Bounds(Power10.niceString(box.x, dx),
					Power10.niceString(box.x + box.width, dx),
					Power10.niceString(box.y, dy),
					Power10.niceString(box.y + box.height, dy));
			return true;
		}
		return false;
	}

	void selectModel(int i) {
		String s[] = models[i];
		cp.funcArea.setText(s[1]);
		cp.Bounds(s[2], s[3], s[4], s[5]);
		cp.nodesField.setText(s[6]);
		reset();
	}

	SoPolygon makeQuadrilateral(double x, double h, Interval y, Interval dy)
			throws ArithmeticException {

		Interval y1 = new Interval(y);
		Interval s = new Interval(dy);
		y1.add(s.mul(h));
		if (!yrange.contains(y) || !yrange.contains(y1))
			throw new ArithmeticException("Out of Range");

		SoPolygon poly = new SoPolygon(4);
		poly.setColor(Color.magenta);
		poly.setFilled(true);
		poly.setOutlined(true);
		poly.x[0] = poly.x[1] = (float) x;
		poly.y[0] = (float) y.inf;
		poly.y[1] = (float) y.sup;
		poly.x[2] = poly.x[3] = (float) (x + h);
		poly.y[2] = (float) y1.sup;
		poly.y[3] = (float) y1.inf;
		return poly;
	}

	public void makeGraph(Expression expression, int nodes, TextArea messageArea) {
		if (expression == null)
			return;
		Interval x = new Interval();
		double h = xrange.width() / nodes;
		double xl = xrange.inf;
		boolean complete = true;

		for (int k = 0; k < nodes; k++) {
			double xr = xl + h;
			if (xl > xrange.sup)
				break;
			if (xr > xrange.sup)
				xr = xrange.sup;

			x.set(xl, xr);
			CenteredInterval xc = x.centered();

			try {
				expression.eval(xc.center);
				Interval y = new Interval(expression.value());
				expression.eval(x);
				Interval dy = new Interval(expression.derivative());

				SoPolygon poly1 = makeQuadrilateral(xc.center, -xc.radius, y,
						dy);
				SoPolygon poly2 = makeQuadrilateral(xc.center, xc.radius, y, dy);
				canvas.addNode(poly1);
				canvas.addNode(poly2);

			} catch (ArithmeticException e) {
				complete = false;
				if (complete)
					messageArea.append("" + e + "\n");
				// e.printStackTrace(System.out);
				SoPolygon poly = new SoPolygon(4);
				poly.setColor(Color.blue);
				poly.setFilled(true);
				poly.setOutlined(true);
				poly.x[0] = poly.x[1] = (float) xl;
				poly.y[0] = poly.y[3] = (float) yrange.inf;
				poly.y[1] = poly.y[2] = (float) yrange.sup;
				poly.x[2] = poly.x[3] = (float) xr;
				canvas.addNode(poly);
			}
			xl = xr;
		}
		if (complete)
			messageArea.append("Graphing successful.");
		else
			messageArea.append("Graphing incomplete.");
	}

	void reset() {
		cp.messageArea.setText("");
		try {
			xrange.parseString(cp.xRangeField.getText());
		} catch (NumberFormatException e) {
			cp.messageArea.setText("x Range:\n" + e.toString());
			return;
		}
		try {
			yrange.parseString(cp.yRangeField.getText());
		} catch (NumberFormatException e) {
			cp.messageArea.setText("y Range:\n" + e.toString());
			return;
		}
		int nodes = 0;
		try {
			nodes = Integer.valueOf(cp.nodesField.getText()).intValue();
			if (nodes < 1)
				throw new NumberFormatException("\nInvalid Node Number");
		} catch (NumberFormatException e) {
			cp.messageArea.setText("Number of Nodes:\n" + e.toString());
			return;
		}

		if (expression == null || !funcdef.equals(cp.funcArea.getText())) {
			funcdef = new String(cp.funcArea.getText());
			String s[] = { "x" };
			try {
				expression = new Expression(funcdef, s);
				cp.messageArea.setText("Parsing successful.\n");
			} catch (Exception e) {
				cp.messageArea.setText("Syntax Error:\n" + e.toString());
				expression = null;
				return;
			}
		}

		canvas.removeAllNodes();
		canvas.setXRange((float) xrange.inf, (float) xrange.sup);
		canvas.setYRange((float) yrange.inf, (float) yrange.sup);
		makeGraph(expression, nodes, cp.messageArea);
		canvas.touch();
	}

	private int count = 0;

	public void paint(Graphics g) {
		if (count == 0) {
			count++;
			repaint(100);
		} else if (count == 1) {
			count++;
			invalidate();
			validate();
		}
	}
}
