package decatur.de.rigorous.plot;

import java.util.Vector;
import java.awt.Insets;

import decatur.de.rigorous.numutil.Power10;

public class SoAxis extends SoShape {

	public final static int AUTOMATIC = 0;
	public final static int MANUAL = 1;
	public final static int NONE = 2;

	boolean doOrigin = false;
	boolean tickValid = false;

	protected SbBox2f bb;
	protected SbBox2f range;

	Vector<Tick> tickX, tickY;

	int xtickmode = AUTOMATIC;
	int ytickmode = AUTOMATIC;

	private Insets insets;
	private String title = null;

	int frameSpacing = 4;
	int labelSpacing = 3;
	int tickLength = 5;

	SbMatrix matrix;

	public SoAxis() {
		super();
		tickX = new Vector<Tick>();
		tickY = new Vector<Tick>();
		bb = new SbBox2f();
		range = new SbBox2f();
		matrix = new SbMatrix();
	}

	public SbMatrix getMatrix() {
		return matrix;
	}

	private void findTicks(float t, float dt, Vector<Tick> v) {
		v.removeAllElements();

		Power10 p = new Power10();
		p.nicestIn(t, t + dt);
		// System.out.println ("t = "+t+" dt = "+dt+" p ="+p);

		Power10 dp = new Power10();
		dp.set(dt, 1);
		int m = (int) dp.m;

		if (m < 15)
			dp.m = 2;
		else if (m < 30)
			dp.m = 5;
		else if (m < 80)
			dp.m = 10;
		else if (m < 100)
			dp.m = 20;
		dp.reduce();

		Power10 q = new Power10();

		q.set(p);
		// System.out.println ("TT: "+q.m+" "+q.e);
		while (q.floatValue() <= t + dt) {
			// System.out.println ("Tick: "+q);
			v.addElement(new Tick(q.floatValue(), q.toString()));
			q.add(dp);
		}

		q.set(p);
		dp.neg();
		q.add(dp);
		// System.out.println ("TT: "+q+" "+p+" "+dp);
		// System.out.println ("TT: "+q.m+" "+q.e);
		while (q.floatValue() >= t) {
			// System.out.println ("Tick: "+q);
			v.addElement(new Tick(q.floatValue(), q.toString()));
			q.add(dp);
		}
	}

	private int maxLabelWidth(Device dev, Vector<Tick> v) {
		int width = 0;
		for (int i = 0; i < v.size(); i++)
			width = Math.max(width,
					dev.stringWidth((v.elementAt(i)).label));
		return width;
	}

	private void makeTicks(Device dev) {

		if (!tickValid) {
			if (xtickmode == AUTOMATIC)
				findTicks(range.x, range.width, tickX);

			if (ytickmode == AUTOMATIC)
				findTicks(range.y, range.height, tickY);

			tickValid = true;
		}

		if (tickX.size() != 0) {
			int w = maxLabelWidth(dev, tickX);
			insets.bottom += dev.getAscent();
			insets.right += w / 2;
			if (tickY.size() == 0)
				insets.left += w / 2;
		}

		if (tickY.size() != 0) {
			int w = maxLabelWidth(dev, tickY);
			insets.left += w;
			insets.top += dev.getAscent() / 2;
			if (tickX.size() == 0)
				insets.bottom += w / 2;
		}
	}

	public void setBoundingBox(float x, float y, float width, float height) {
		bb.reshape(x, y, width, height);
	}

	public void setRange(float x, float y, float width, float height) {
		if (x != range.x || y != range.y || width != range.width
				|| height != range.height) {
			tickValid = false;
			range.reshape(x, y, width, height);
		}
	}

	public void setTitle(String s) {
		if (s == null)
			title = null;
		else
			title = new String(s);
	}

	public String getTitle() {
		return title;
	}

	public void doOrigin(boolean flag) {
		doOrigin = flag;
	}

	public void setYmode(int mode) {
		ytickmode = mode;
	}

	public void setXmode(int mode) {
		xtickmode = mode;
	}

	public void addTickX(float pos, String s) {
		tickX.addElement(new Tick(pos, s));
	}

	public void addTickY(float pos, String s) {
		tickY.addElement(new Tick(pos, s));
	}

	private void drawTick(Device dev, Vector<Tick> v, int i) {
		Tick t = v.elementAt(i);

		if (v == tickX) {
			dev.drawString(t.label, t.pos,
					range.y - dev.idy(dev.getAscent() + labelSpacing),
					Device.CENTER);
			dev.drawLine(t.pos, range.y, t.pos, range.y + dev.idy(tickLength));
		} else if (v == tickY) {
			dev.drawString(t.label, range.x - dev.idx(labelSpacing), t.pos
					- dev.idy(dev.getAscent()) / 2, Device.LEFT);
			dev.drawLine(range.x, t.pos, range.x + dev.idx(tickLength), t.pos);
		}
	}

	public void paint(Device dev) {
		int sp = frameSpacing + labelSpacing;
		insets = new Insets(sp, sp, sp, sp);

		/*
		 * int color = dev.getColor(); dev.setColor (0xffffff); dev.fillRect
		 * (bb.x, bb.y, bb.width, bb.height); dev.setColor (color);
		 */
		// dev.drawRect (bb.x, bb.y, bb.width, bb.height);

		dev.setFilled(false);
		dev.setOutlined(false);

		if (title != null) {
			dev.drawString(title, bb.x + bb.width / 2,
					bb.y + bb.height - dev.idy(dev.getAscent() + frameSpacing),
					Device.CENTER);
			insets.top += dev.getAscent() + labelSpacing;
		}

		makeTicks(dev);

		float sx = (bb.width - dev.idx(insets.left + insets.right))
				/ range.width;
		float sy = (bb.height - dev.idy(insets.top + insets.bottom))
				/ range.height;

		dev.matrix.translate(bb.x + dev.idx(insets.left) - sx * range.x, bb.y
				+ dev.idy(insets.bottom) - sy * range.y);
		dev.matrix.scale(sx, sy);

		matrix.set(dev.matrix);

		if (doOrigin) {
			if (range.y < 0.0f && range.y + range.height > 0.0f)
				dev.drawLine(range.x, 0.0f, range.x + range.width, 0.0f);
			if (range.x < 0.0f && range.x + range.width > 0.0f)
				dev.drawLine(0.0f, range.y, 0.0f, range.y + range.height);
		}

		dev.drawRect(range.x, range.y, range.width, range.height);

		for (int i = 0; i < tickX.size(); i++)
			drawTick(dev, tickX, i);

		for (int i = 0; i < tickY.size(); i++)
			drawTick(dev, tickY, i);

		dev.clipRect(range.x, range.y, range.width, range.height);
	}

}
