package decatur.de.rigorous.plot;

import java.awt.*;
import java.io.*;

import decatur.de.rigorous.numutil.Power10;

public class SoAxisCanvas extends Canvas {
	private int debug = 0;

	public final static int AUTOMATIC = 0;
	public final static int NONAUTOMATIC = 1;

	private AwtDevice dev;
	private SoSeparator root;
	private SoAxis axis;
	private SoTransform map;
	private SoGroup userRoot;

	private int modeX = AUTOMATIC;
	private int modeY = AUTOMATIC;

	public float xMin, xMax, yMin, yMax; // The range.
	boolean isValid = false;

	Image iBuffer;

	Rectangle box = null;
	Point currentPointer, downPointer;
	SbVec2f point;
	boolean focus = false;
	SbMatrix matrix = null;

	SoRenderAction renderAction;

	int width, height;

	public SoAxisCanvas(int width, int height) {
		this.width = width;
		this.height = height;
		setSize(width, height);
		dev = new AwtDevice();
		renderAction = new SoRenderAction();
		point = new SbVec2f();
		currentPointer = new Point(0, 0);
		downPointer = new Point(0, 0);
		root = new SoSeparator();
		axis = new SoAxis();
		axis.doOrigin(true);
		map = new SoTransform();
		root.addNode(map);
		root.addNode(axis);
		userRoot = new SoGroup();
		root.addNode(userRoot);
	}

	public SbVec2f getResolution() {
		if (matrix == null)
			return null;
		SbVec2f p = new SbVec2f();
		matrix.idtransform(p, 1, 1);
		p.x = Math.abs(p.x);
		p.y = Math.abs(p.y);
		return p;
	}

	public void setTitle(String s) {
		axis.setTitle(s);
	}

	public void setXRange(int mode) {
		modeX = mode;
	}

	public void setXRange(float min, float max) {
		xMin = min;
		xMax = max;
		modeX = NONAUTOMATIC;
	}

	public void setYRange(int mode) {
		modeY = mode;
	}

	public void setYRange(float min, float max) {
		yMin = min;
		yMax = max;
		modeY = NONAUTOMATIC;
	}

	public void addNode(SoNode p) {
		userRoot.addNode(p);
	}

	public void removeAllNodes() {
		userRoot.removeAllNodes();
	}

	public boolean removeNode(SoNode p) {
		return userRoot.removeNode(p);
	}

	public boolean handleEvent(Event e) {
		if (e.shiftDown() && e.id == Event.MOUSE_DOWN) {
			print();
			return true;
		} else if (e.id == Event.MOUSE_EXIT) {
			focus = false;
			repaint();
			return true;
		} else if (e.id == Event.MOUSE_ENTER) {
			focus = true;
			return true;
		} else if (e.id == Event.MOUSE_DOWN) {
			currentPointer.move(e.x, e.y);
			downPointer.move(e.x, e.y);
			repaint();
			return true;
		} else if (e.id == Event.MOUSE_UP) {
			currentPointer.move(e.x, e.y);
			if (matrix == null)
				return true;
			if (e.x == downPointer.x && e.y == downPointer.y) {
				SbVec2f p = new SbVec2f();
				matrix.itransform(p, e.x, e.y);
				e.arg = p;
			} else {
				SbBox2f sbbox = new SbBox2f();
				matrix.itransform(point, downPointer.x, downPointer.y);
				sbbox.x = point.x;
				sbbox.y = point.y;
				matrix.idtransform(point, currentPointer.x - downPointer.x,
						currentPointer.y - downPointer.y);
				sbbox.width = point.x;
				sbbox.height = point.y;
				e.arg = sbbox;
			}
			box = null;
			repaint();
			// Process event in parent container
		} else if (e.id == Event.MOUSE_DRAG) {
			currentPointer.move(e.x, e.y);
			if (box == null)
				box = new Rectangle();
			box.x = e.x <= downPointer.x ? e.x : downPointer.x;
			box.width = Math.abs(e.x - downPointer.x);
			box.y = e.y <= downPointer.y ? e.y : downPointer.y;
			box.height = Math.abs(e.y - downPointer.y);
			repaint();
			return true;
		} else if (e.id == Event.MOUSE_MOVE) {
			currentPointer.move(e.x, e.y);
			repaint();
			return true;
		}
		return super.handleEvent(e);
	}

	private void makeRange() {
		if (modeX == AUTOMATIC || modeY == AUTOMATIC) {
			BoundingBoxDevice dev = new BoundingBoxDevice();
			renderAction.setDevice(dev);
			renderAction.apply(userRoot);
			if (modeX == AUTOMATIC) {
				xMin = dev.xMin;
				xMax = dev.xMax;
			}
			if (modeY == AUTOMATIC) {
				yMin = dev.yMin;
				yMax = dev.yMax;
			}
		}

		if (debug > 0)
			System.out.println("plot.AxisCanvas " + axis.getTitle()
					+ "\nBounding Box:\n" + xMin + " " + xMax + " " + yMin
					+ " " + yMax + "\n Modes: " + modeX + " " + modeY);

		axis.setRange(xMin, yMin, xMax - xMin, yMax - yMin);
	}

	public void touch() {
		isValid = false;
		repaint();
	}

	public void print() {
		FileDialog fd = new FileDialog(new Frame(), "Print to file",
				FileDialog.SAVE);
		fd.setVisible(true);

		try {
			File f = new File(fd.getDirectory(), fd.getFile());
			print(new PrintStream(new FileOutputStream(f)));
		} catch (Exception e) {
			e.printStackTrace(System.out);
		}
	}

	public void print(PrintStream stream) {
		map.setScale(1, 1);
		map.setTranslate(0, 0);

		int width = 300;
		int height = 300;
		axis.setBoundingBox(0.0f, 0.0f, width, height);
		makeRange();

		PostScriptDevice devPS = new PostScriptDevice(stream, "test", width,
				height);
		// devPS.setFont ("Helvetica", 30);
		renderAction.setDevice(devPS);
		renderAction.apply(root);
		devPS.close();
	}

	public void update(Graphics g) {
		paint(g);
	}

	public void paint(Graphics g) {
		Dimension size = getSize();
		
		if (width != size.width || height != size.height) {
			width = size.width;
			height = size.height;
			isValid = false;
		}

		if (!isValid) {
			// Get new GC every time to reset clipping
			iBuffer = createImage(width, height);
			paint1(iBuffer.getGraphics());
			isValid = true;
		}

		g.drawImage(iBuffer, 0, 0, this);

		if (box != null) {
			g.setColor(Color.red);
			g.drawRect(box.x, box.y, box.width, box.height);
		}

		if (focus) {
			SbVec2f resolution = getResolution();
			matrix.itransform(point, currentPointer.x, currentPointer.y);
			g.setColor(Color.red);
			String s = "(" + Power10.niceString(point.x, resolution.x * 0.1)
					+ ", " + Power10.niceString(point.y, resolution.y * 0.1)
					+ ")";
			FontMetrics fm = g.getFontMetrics();
			int l = fm.stringWidth(s);
			g.drawString(s, Math.min(currentPointer.x, width - l),
					currentPointer.y);
		}

	}

	public void paint1(Graphics g) {

		g.setColor(Color.white);
		g.fillRoundRect(0, 0, width, height, 5, 5);

		g.setColor(Color.black);
		g.drawRoundRect(1, 1, width - 3, height - 3, 5, 5);
		g.setColor(Color.red);
		g.drawRoundRect(0, 0, width - 1, height - 1, 5, 5);

		map.setScale(1, -1);
		map.setTranslate(0, height);

		axis.setBoundingBox(1, 1, width - 2, height - 2);
		makeRange();

		g.setColor(Color.black);
		dev.setContext(g);
		renderAction.setDevice(dev);
		renderAction.apply(root);
		matrix = axis.getMatrix();
		isValid = true;
	}
}
