package decatur.de.rigorous.plot;

import java.awt.*;
import java.util.Vector;

public class TextCanvas extends Canvas {
	public Vector<String[]> text;
	Font font;
	int delta = 0;
	int style = Font.PLAIN;
	int fontsize = 0;
	String format;
	int width = 10, height = 10;

	public TextCanvas() {
		text = new Vector<String[]>();
		// resize (w, h);
		// validate ();
	}

	public Dimension preferredSize() {
		return new Dimension(width, height);
	}

	public Dimension minimumSize() {
		return preferredSize();
	}

	public void incFontSize(int delta) {
		this.delta = delta;
		repaint();
	}

	public void setFontStyle(int style) {
		this.style = style;
		repaint();
	}

	public void setFontSize(int fontsize) {
		this.fontsize = fontsize;
		repaint();
	}

	public void setFormat(String s) {
		format = s;
		repaint();
	}

	public void addRow(String s) {
		int ncolumns = format.length();
		String[] a = new String[ncolumns];
		int k = 0, l;
		for (int j = 0; j < ncolumns; j++, k = l + 1) {
			l = s.indexOf('&', k);
			if (l == -1) {
				a[j] = new String(s.substring(k));
				break;
			} else
				a[j] = new String(s.substring(k, l));
		}
		text.addElement(a);
		repaint();
	}

	String getCell(int i, int j) {
		return (text.elementAt(i))[j];
	}

	public void paint(Graphics g) {
		int w = 0;
		int h = 0;
		int ncolumns = format.length();
		Font f = g.getFont();
		if (fontsize == 0)
			fontsize = f.getSize() + delta;

		if (font == null || font.getSize() != fontsize
				|| font.getStyle() != style)
			font = new Font(f.getName(), style, fontsize);

		g.setFont(font);
		FontMetrics fm = g.getFontMetrics();

		int dy = fm.getMaxAscent() + fm.getMaxDescent();
		int x = 0;
		int max;
		for (int j = 0; j < ncolumns; j++, x += max + fm.charWidth('X')) {
			int y = dy;
			int i;
			for (max = 0, i = 0; i < text.size(); i++)
				max = Math.max(max, fm.stringWidth(getCell(i, j)));

			for (i = 0; i < text.size(); i++, y += dy) {
				int off = 0;
				int stringwidth = fm.stringWidth(getCell(i, j));

				if (format.charAt(j) == 'r')
					off = max - stringwidth;
				else if (format.charAt(j) == 'c')
					off = (max - stringwidth) / 2;
				g.drawString(getCell(i, j), x + off, y);
				w = Math.max(w, x + off + stringwidth);
				h = Math.max(h, y);
			}
		}
		if (h != height || w != width) {
			width = w;
			height = h;
			invalidate();
		}
	}
}
