package decatur.de.rigorous.plot2d;

import java.awt.*;
import java.io.*;

class PostScript {
	static String FINDFONT = " findfont ", SETFONT = " setfont ",
			SCALEFONT = " scalefont ", DIV = " div ", DUP = " dup ",
			EXCH = " exch ", POP = " pop ", STRINGWIDTH = " stringwidth ",
			TRANSLATE = " translate ", MOVETO = " mv ", SHOW = " show ",
			NEWPATH = " n ", CLOSEPATH = " c ", FILL = " f ", STROKE = " s ",
			CURVETO = " curveto ", LINETO = " l ", GRESTORE = " grestore ",
			GSAVE = " gsave ", SETRGBCOLOR = " setrgbcolor ";

	PrintStream stream = null;

	void stream(PrintStream stream) {
		if (stream == null && this.stream != null)
			this.stream.close();
		this.stream = stream;
	}

	public void issue(String s) {
		if (stream != null)
			stream.println(s);
	}

}

public class PlotDevice {
	Image image;
	Graphics offg;
	int devwidth, devheight;
	int ref_width, ascent, descent;
	PostScript ps = new PostScript();
	static int CENTER = 0, RIGHT = 1, LEFT = 2;
	FontMetrics fm = null;

	PlotDevice(Canvas canvas, int devwidth, int devheight) {
		image = canvas.createImage(devwidth, devheight);
		offg = image.getGraphics();
		offg.setPaintMode();
		this.devwidth = devwidth;
		this.devheight = devheight;

		try {
			fm = offg.getFontMetrics();
			ref_width = fm.stringWidth("XXXXX") / 5;
			ascent = fm.getAscent();
			descent = fm.getDescent();
		} catch (NullPointerException e) {
			ref_width = ascent = 12;
			descent = 3;
		}

	}

	public void close() {
	}

	public void PSopen(PrintStream stream) {
		ps.stream(stream);
		ps.issue("%!PS-Adobe-2.0 EPSF-2.1");
		ps.issue("%%Title: none");
		ps.issue("%%Creator: PlotDevice.java Version 0.1");
		ps.issue("%%CreationDate: " + new java.util.Date());
		ps.issue("%%Orientation: Portrait");
		ps.issue("%%BoundingBox: 0 0 " + devwidth + " " + devheight);
		ps.issue("%%Pages: 0");
		ps.issue("%%BeginSetup");
		ps.issue("%%EndSetup");
		ps.issue("%%EndComments");
		ps.issue("/pjdict 20 dict def pjdict begin");
		ps.issue("/mv {moveto} bind def");
		ps.issue("/l {lineto} bind def");
		ps.issue("/n {newpath} bind def");
		ps.issue("/c {closepath} bind def");
		ps.issue("/s {stroke} bind def");
		ps.issue("/f {fill} bind def");
		ps.issue(PostScript.GSAVE);
		ps.issue("/Helvetica" + PostScript.FINDFONT + PostScript.DUP + PostScript.SETFONT);
		ps.issue("(XXXXX)" + PostScript.STRINGWIDTH + PostScript.POP + 5 + PostScript.DIV + ref_width
				+ PostScript.EXCH + PostScript.DIV + PostScript.SCALEFONT + PostScript.SETFONT);
	}

	public void PSclose() {
		ps.issue("grestore end");
		ps.stream(null);
	}

	public void open() {
		offg.setColor(Color.white);
		offg.fillRect(0, 0, devwidth, devheight);
	}

	public Image getImage() {
		return image;
	}

	public int stringWidth(String s) {
		return s.length() * ref_width;
	}

	public int getAscent() {
		return ascent;
	}

	public int getDescent() {
		return descent;
	}

	public void drawString(String s, int x, int y, int align) {
		int width;
		if (fm != null)
			width = fm.stringWidth(s);
		else
			width = stringWidth(s);
		int offset = 0;
		if (align == CENTER)
			offset = -width / 2;
		else if (align == RIGHT)
			offset = -width;
		if (x + offset >= 0 && x + offset + width <= devwidth)
			offg.drawString(s, x + offset, devheight - y);

		ps.issue("(" + s + ") dup stringwidth pop ");
		if (align == CENTER)
			ps.issue("2 div");
		else if (align == LEFT)
			ps.issue("pop 0");

		ps.issue("" + x + " exch sub " + y + " moveto show");
	}

	public void drawLine(int x1, int y1, int x2, int y2) {
		offg.drawLine(x1, devheight - y1, x2, devheight - y2);
		ps.issue(PostScript.NEWPATH);
		ps.issue("" + x1 + " " + y1 + PostScript.MOVETO);
		ps.issue("" + x2 + " " + y2 + PostScript.LINETO);
		ps.issue(PostScript.CLOSEPATH);
		ps.issue(PostScript.STROKE);
	}

	public void drawRect(int x, int y, int width, int height) {
		offg.drawRect(x, devheight - y - height, width, height);
		ps.issue(PostScript.NEWPATH);
		ps.issue("" + x + " " + y + PostScript.MOVETO);
		ps.issue("" + (x + width) + " " + y + PostScript.LINETO);
		ps.issue("" + (x + width) + " " + (y + height) + PostScript.LINETO);
		ps.issue("" + x + " " + (y + height) + PostScript.LINETO);
		ps.issue(PostScript.CLOSEPATH);
		ps.issue(PostScript.STROKE);
	}

	public void fillRect(int x, int y, int width, int height) {
		offg.fillRect(x, devheight - y - height, width, height);

		ps.issue(PostScript.NEWPATH);
		ps.issue("" + x + " " + y + PostScript.MOVETO);
		ps.issue("" + (x + width) + " " + y + PostScript.LINETO);
		ps.issue("" + (x + width) + " " + (y + height) + PostScript.LINETO);
		ps.issue("" + x + " " + (y + height) + PostScript.LINETO);
		ps.issue(PostScript.CLOSEPATH);
		ps.issue(PostScript.FILL);
	}

	public void setColor(Color c) {
		offg.setColor(c);
		ps.issue("" + (c.getRed() / 255) + " " + (c.getGreen() / 255) + " "
				+ (c.getBlue() / 255) + PostScript.SETRGBCOLOR);
	}

	public void drawOval(int x, int y, int width, int height, boolean fill) {
		if (fill)
			offg.fillOval(x, devheight - y - height, width, height);
		else
			offg.drawOval(x, devheight - y - height, width, height);

		ps.issue(PostScript.NEWPATH + x + " " + (y + height / 2) + PostScript.MOVETO);
		ps.issue("" + x + " " + (y + height) + " " + x + " " + (y + height)
				+ " " + (x + width / 2) + " " + (y + height) + PostScript.CURVETO
				+ PostScript.CLOSEPATH);
		if (fill)
			ps.issue(PostScript.FILL);
		else
			ps.issue(PostScript.STROKE);
	}

	public void fillPath(Polygon path) {
		for (int i = 0; i < path.npoints; i++)
			path.ypoints[i] = devheight - path.ypoints[i];

		offg.fillPolygon(path);
		for (int i = 0; i < path.npoints; i++) {
			int j = (i + 1) % path.npoints;
			offg.drawLine(path.xpoints[i], path.ypoints[i], path.xpoints[j],
					path.ypoints[j]);
		}
		for (int i = 0; i < path.npoints; i++)
			path.ypoints[i] = devheight - path.ypoints[i];

		ps.issue(PostScript.NEWPATH);

		for (int i = 0; i < path.npoints; i++) {
			ps.issue("" + path.xpoints[i] + " " + path.ypoints[i]);
			if (i == 0)
				ps.issue(PostScript.MOVETO);
			else
				ps.issue(PostScript.LINETO);
		}
		ps.issue(PostScript.CLOSEPATH + PostScript.FILL);
	}

}
