package decatur.de.rigorous.plot2d;

import java.awt.*;
import java.applet.Applet;

import decatur.de.rigorous.numutil.*;
import decatur.de.rigorous.parse.*;

class MyPanel extends BoxedPanel {
	TextField xmin, xmax, ymin, ymax, nodes;
	TextArea message, func, root;
	Button eval, erase;
	Choice choice;
	TestPlot2d applet;

	MyPanel(TestPlot2d applet) {
		super();
		this.applet = applet;

		xmin = new TextField("", 15);
		xmax = new TextField("", 15);
		ymin = new TextField("", 15);
		ymax = new TextField("", 15);
		eval = new Button("Evaluate");
		erase = new Button("Clear");
		choice = new Choice();

		root = new TextArea("", 3, 25);
		func = new TextArea("[0.5,0.55]*x^3-x^2", 4, 25);
		nodes = new TextField("40", 4);
		message = new TextArea("Message area", 4, 25);

		add(3, 2, 0, 0, message);

		add(2, 1, 3, 0, choice);
		add(1, 1, 3, 1, new Label("f(x) :=", Label.RIGHT));
		add(3, 2, 4, 1, func);

		add(1, 1, 0, 3, new Label("min x =", Label.RIGHT));
		add(2, 1, 1, 3, xmin);

		add(1, 1, 3, 3, new Label("max x =", Label.RIGHT));
		add(2, 1, 4, 3, xmax);

		add(1, 1, 0, 4, new Label("min y =", Label.RIGHT));
		add(2, 1, 1, 4, ymin);

		add(1, 1, 3, 4, new Label("max y =", Label.RIGHT));
		add(2, 1, 4, 4, ymax);

		add(1, 1, 0, 5, new Label("root =", Label.RIGHT));
		add(2, 2, 1, 5, root);

		add(1, 1, 3, 5, new Label("nodes =", Label.RIGHT));
		add(1, 1, 4, 5, nodes);
		add(1, 1, 4, 6, eval);

		setVisible(true);
	}

	public Dimension preferredSize() {
		return new Dimension(applet.getSize().width - 10,
				applet.getSize().height - 400 - 10);
	}

	public synchronized Dimension minimumSize() {
		return new Dimension(500, 300);
	}

	public void Bounds(String infx, String supx, String infy, String supy) {
		xmin.setText(infx);
		xmax.setText(supx);
		ymin.setText(infy);
		ymax.setText(supy);
	}
}

class MyPlot extends Plot2d {

	public MyPlot(int devwidth, int devheight) {
		super(devwidth, devheight);
	}

	boolean slope(double x, double h, Interval y, Interval dy) {
		if (y.inf < infy - res[1] || y.sup > supy + res[1]
				|| dy.sup * Math.abs(h) + y.sup > supy + res[1]
				|| dy.inf * Math.abs(h) + y.inf < infy - res[1]) {
			setColor(Color.blue.brighter());
			newpath(x, infy);
			lineto(x, supy);
			lineto(x + h, supy);
			lineto(x + h, infy);
			fillpath();
			return false;
		}
		setColor(Color.magenta);
		newpath(x, y.inf);
		lineto(x, y.sup);
		lineto(x + h, +dy.sup * Math.abs(h) + y.sup);
		lineto(x + h, +dy.inf * Math.abs(h) + y.inf);
		fillpath();
		return true;
	}

	public void graph(Expression expression, int nodes, TextArea message) {
		erase();
		if (expression == null)
			return;
		Interval a = new Interval();
		double h = (supx - infx) / nodes;
		Interval dy = Interval.ZERO;
		boolean valid = false;
		boolean inrange = true;
		for (int k = 0; k <= nodes; k++) {
			double x = infx + k * h;
			try {
				expression.eval(x);
				Interval y = new Interval(expression.value());
				if (valid) {
					dy.neg();
					if (!slope(x, -h / 2, y, dy))
						inrange = false;
				}
				if (k < nodes) {
					valid = true;
					a.set(x, infx + (k + 1) * h);
					expression.eval(a);
					dy = new Interval(expression.derivative());
					if (!slope(x, h / 2, y, dy))
						inrange = false;
				}
			} catch (ArithmeticException e) {
				message.setText("" + e + "\n");
				valid = false;
				inrange = false;
				setColor(Color.blue);
				drawRect(x, infy, h, supy - infy, true);
			}
		}
		if (!inrange)
			message.append("Graphing incomplete.");
		else
			message.append("Graphing successful.");
		setColor(Color.black);
		drawAxes();
	}

	public void paint(Graphics g) {
		super.paint(g);
	}
}

public class TestPlot2d extends Applet {
	MyPlot plot;
	MyPanel cp;
	Expression expression = null;
	String funcdef = "";
	String[][] models = {
			{ "cubic", "[0.5,0.55]*x^3-2*x", "-3", "3", "-10", "10", "40" },
			{ "trig1", "sin(x*[1,1.03])", "0", "40", "-2", "2", "150" },
			{ "singular", "1/(x^2-0.2)", "-2", "2", "-20", "20", "100" },
			{ "trig2", "x*sin(1/x)", "0", "0.1", "-3", "3", "80" },
			{ "identity", "sin(x)^2+cos(x)^2", "0", "20", "0", "2", "80" } };

	RootFinder rootfinder;

	public void init() {
		rootfinder = new RootFinder();
		plot = new MyPlot(400, 400);
		plot.setDoOrigin(true);
		cp = new MyPanel(this);
		for (int i = 0; i < models.length; i++)
			cp.choice.addItem(models[i][0]);
		add(plot);
		add(cp);
		selectModel(0);
		validate();
	}

	public boolean action(Event evt, Object what) {
		if (evt.target == cp.eval) {
			reset();
			return true;
		} else if (evt.target == cp.choice) {
			selectModel(cp.choice.getSelectedIndex());
			return true;
		}
		return false;
	}

	public boolean mouseUp(Event evt, int xdummy, int ydummy) {
		System.out.println("Mouse Up in applet");
		if (evt.arg.getClass().getName().equals("[D")) {
			double a[] = (double[]) evt.arg;
			if (a.length == 2) {
				try {
					Interval x = rootfinder.find(a[0], expression);
					cp.message.setText("Enclosure for root found!");
					cp.root.setText("[" + x.inf + ",\n " + x.sup + "]");
					double xm = x.mid();
					double rad = Math.max(x.rad(), plot.res[0] * 3);
					// System.out.println("rad: "+plot.res[0]+" "+plot.res[1]);
					plot.drawOval(xm - rad, -plot.res[1] * 3, 2 * rad,
							plot.res[1] * 6, true);
					plot.repaint();
				} catch (ArithmeticException e) {
					cp.message.setText("" + e);
					cp.root.setText("");
				}
				return true;
			} else if (a.length == 4) {
				double dx = (a[2] - a[0]) * 0.01;
				double dy = (a[3] - a[1]) * 0.01;
				cp.Bounds(Double10.format(a[0] - dx, a[0]),
						Double10.format(a[2], a[2] + dx),
						Double10.format(a[1] - dy, a[1]),
						Double10.format(a[3], a[3] + dy));
				return true;
			}
		}
		return false;
	}

	void selectModel(int i) {
		String s[] = models[i];
		cp.func.setText(s[1]);
		cp.Bounds(s[2], s[3], s[4], s[5]);
		cp.nodes.setText(s[6]);
		reset();
	}

	void reset() {
		cp.message.setText("");
		double inf_x = Double.valueOf(cp.xmin.getText()).doubleValue();
		double sup_x = Double.valueOf(cp.xmax.getText()).doubleValue();
		double inf_y = Double.valueOf(cp.ymin.getText()).doubleValue();
		double sup_y = Double.valueOf(cp.ymax.getText()).doubleValue();
		plot.Bounds(inf_x, sup_x, inf_y, sup_y);
		parsefunc();
		int nodes = Integer.valueOf(cp.nodes.getText()).intValue();
		if (nodes < 1) {
			nodes = 40;
			cp.nodes.setText("" + nodes);
		}
		plot.graph(expression, nodes, cp.message);
		plot.repaint();
	}

	void parsefunc() {
		if (funcdef.equals(cp.func.getText()))
			return;
		funcdef = new String(cp.func.getText());
		String s[] = { "x" };
		try {
			expression = new Expression(funcdef, s);
			cp.message.setText("Parsing successful.\n");
		} catch (Exception e) {
			cp.message.setText("Syntax Error:\n" + e.toString());
			expression = null;
		}
	}

}
